@@ -2,6 +2,18 @@
 Common utilities
 '''
 import re
+import os
+import time
+import sys
+import struct
+
+try:
+    basestring = basestring
+except NameError:
+    basestring = (str, bytes)
+
+AF_PIPE = 255  # Right now AF_MAX == 40
+
 
 size_suffixes = {'b': 1,
                  'k': 1024,
@@ -118,9 +130,44 @@ def map_namespace(prefix, ns):
     return (by_name, by_value)
 
 
+def fnv1(data):
+    '''
+    FNV1 -- 32bit hash
+
+    @param data: input
+    @type data: bytes
+
+    @return: 32bit int hash
+    @rtype: int
+
+    See: http://www.isthe.com/chongo/tech/comp/fnv/index.html
+    '''
+    hval = 0x811c9dc5
+    for i in range(len(data)):
+        hval *= 0x01000193
+        hval ^= struct.unpack('B', data[i])[0]
+    return hval & 0xffffffff
+
+
+def uuid32():
+    '''
+    Return 32bit UUID, based on the current time and pid.
+
+    @return: 32bit int uuid
+    @rtype: int
+    '''
+    return fnv1(struct.pack('QQ',
+                            int(time.time() * 1000),
+                            os.getpid()))
+
+
 def hexdump(payload, length=0):
     '''
     Represent byte string as hex -- for debug purposes
     '''
-    return ':'.join('{0:02x}'.format(ord(c))
-                    for c in payload[:length] or payload)
+    if sys.version[0] == '3':
+        return ':'.join('{0:02x}'.format(c)
+                        for c in payload[:length] or payload)
+    else:
+        return ':'.join('{0:02x}'.format(ord(c))
+                        for c in payload[:length] or payload)
