@@ -13,7 +13,11 @@ Quick start:
 import uuid
 import platform
 import threading
-from Queue import Empty
+try:
+    from Queue import Empty
+except ImportError:
+    from queue import Empty
+
 from socket import AF_INET
 from socket import AF_INET6
 from pyroute2.common import Dotkeys
@@ -455,7 +459,6 @@ class Interface(Transactional):
             self['ports'] = LinkedSet()
             for i in nla_fields:
                 self[i] = None
-            self['flags'] = 0
             for i in ('state', 'change', 'mask'):
                 del self[i]
         # 8<-----------------------------------
@@ -513,9 +516,14 @@ class Interface(Transactional):
 
     @update
     def add_ip(self, direct, ip, mask=None):
+        # split mask
         if mask is None:
             ip, mask = ip.split('/')
             mask = int(mask, 0)
+        # FIXME: make it more generic
+        # skip IPv6 link-local addresses
+        if ip[:4] == 'fe80' and mask == 64:
+            return
         if not direct:
             transaction = self.last()
             transaction.add_ip(ip, mask)
@@ -575,7 +583,7 @@ class Interface(Transactional):
         Commit transaction. In the case of exception all
         changes applied during commit will be reverted.
         '''
-        e = None
+        error = None
         added = None
         removed = None
         if tid:
@@ -665,7 +673,7 @@ class Interface(Transactional):
                     request[key] = added[key]
 
             # apply changes only if there is something to apply
-            if request:
+            if any([request[x] is not None for x in request]):
                 self.nl.link('set', index=self['index'], **request)
 
             # 8<---------------------------------------------
@@ -720,7 +728,17 @@ class Interface(Transactional):
                         for i in added['ipaddr']:
                             self['ipaddr'].add(i)
                 # 8<-----------------------------------------
-                self.commit(transaction=snapshot, rollback=True)
+                ret = self.commit(transaction=snapshot, rollback=True)
+                # if some error was returned by the internal
+                # closure, substitute the initial one
+                if ret is not None:
+                    error = ret
+                else:
+                    error = e
+            elif isinstance(e, NetlinkError) and getattr(e, 'code', 0) == 1:
+                # It is <Operation not permitted>, catched in
+                # rollback. So return it -- see ~5 lines above
+                return e
             else:
                 # somethig went wrong during automatic rollback.
                 # that's the worst case, but it is still possible,
@@ -750,9 +768,9 @@ class Interface(Transactional):
             self.reload()
 
         # raise exception for failed transaction
-        if e is not None:
-            e.transaction = transaction
-            raise e
+        if error is not None:
+            error.transaction = transaction
+            raise error
 
     def up(self):
         '''
@@ -783,8 +801,8 @@ class IPDB(Dotkeys):
     No methods of the class should be called directly.
     '''
 
-    def __init__(self, nl=None, host='localsystem', mode='implicit',
-                 key=None, cert=None, ca=None):
+    def __init__(self, nl=None, host=None, mode='implicit',
+                 key=None, cert=None, ca=None, iclass=Interface):
         '''
         Parameters:
             * nl -- IPRoute() reference
@@ -798,6 +816,7 @@ class IPDB(Dotkeys):
         self.nl = nl or IPRoute(host=host, key=key, cert=cert, ca=ca)
         self.mode = mode
         self._stop = False
+        self.iclass = iclass
 
         # resolvers
         self.by_name = Dotkeys()
@@ -863,7 +882,7 @@ class IPDB(Dotkeys):
 
         FIXME: this should be documented.
         '''
-        i = Interface(nl=self.nl, ipdb=self, mode='snapshot')
+        i = self.iclass(nl=self.nl, ipdb=self, mode='snapshot')
         i['kind'] = kind
         i['index'] = kwarg.get('index', 0)
         i['ifname'] = ifname
@@ -888,7 +907,7 @@ class IPDB(Dotkeys):
                 self.by_index[dev['index']] = \
                 self[dev['index']] = \
                 self.get(dev.get_attr('IFLA_IFNAME'), None) or \
-                Interface(nl=self.nl, ipdb=self, mode=self.mode)
+                self.iclass(nl=self.nl, ipdb=self, mode=self.mode)
             i.load(dev)
             self[i['ifname']] = \
                 self.by_name[i['ifname']] = i
